@@ -516,6 +516,7 @@ void         ssl_init_Child(server_rec *, pool *);
 void         ssl_hook_AddModule(module *);
 char        *ssl_hook_RewriteCommand(cmd_parms *, void *config, const char *);
 void         ssl_hook_NewConnection(conn_rec *);
+void         ssl_hook_TimeoutConnection(int);
 void         ssl_hook_CloseConnection(void *);
 int          ssl_hook_Auth(request_rec *);
 int          ssl_hook_Access(request_rec *);
@@ -601,6 +602,7 @@ void         ssl_compat_variables(request_rec *);
 #endif
 
 /*  Utility Functions  */
+char        *ssl_util_vhostid(pool *, server_rec *);
 void         ssl_util_strupper(char *);
 void         ssl_util_uuencode(char *, const char *, BOOL);
 void         ssl_util_uuencode_binary(unsigned char *, const unsigned char *, int, BOOL);
