@@ -157,8 +157,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
      * Remember the connection information for
      * later access inside callback functions
      */
-    ssl_log(srvr, SSL_LOG_INFO, "Connection to child %d established (server %s:%d)",
-            conn->child_num, srvr->server_hostname, srvr->port);
+    ssl_log(srvr, SSL_LOG_INFO, "Connection to child %d established (server %s)",
+            conn->child_num, ssl_util_vhostid(conn->pool, srvr));
 
     /*
      * Create a new SSL connection with the configured server SSL context and
@@ -190,7 +190,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
                     "Unable to configure server certificate for connection");
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
-            fb->flags |= B_EOF|B_EOUT;
+            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+            conn->aborted = 1;
             return;
         }
         if (SSL_use_RSAPrivateKey(ssl, sc->prsaKey) <= 0) {
@@ -198,7 +199,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
                     "Unable to configure server private key for connection");
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
-            fb->flags |= B_EOF|B_EOUT;
+            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+            conn->aborted = 1;
             return;
         }
     }
@@ -209,6 +211,17 @@ void ssl_hook_NewConnection(conn_rec *conn)
     ap_ctx_set(fb->ctx, "ssl:clientdn", NULL);
     ap_ctx_set(fb->ctx, "ssl:verifyerror", NULL);
 
+    /* 
+     * We have to manage a I/O timeout ourself, because Apache
+     * does it the first time when reading the request, but we're
+     * working some time before this happens.
+     */
+    ap_ctx_set(ap_global_ctx, "ssl::handshake::timeout", (void *)FALSE);
+    ap_set_callback_and_alarm(ssl_hook_TimeoutConnection, srvr->timeout);
+
+    /*
+     * Now enter the SSL Handshake Phase
+     */
     while (!SSL_is_init_finished(ssl)) {
 
         if ((rc = SSL_accept(ssl)) <= 0) {
@@ -225,7 +238,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 while (!SSL_shutdown(ssl));
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
-                fb->flags |= B_EOF|B_EOUT;
+                ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+                conn->aborted = 1;
                 return;
             }
 #if (SSL_LIBRARY_VERSION >= 0x0900)
@@ -274,12 +288,38 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 return;
             }
 #endif
+            else if (ap_ctx_get(ap_global_ctx, "ssl::handshake::timeout") == (void *)TRUE) {
+                ssl_log(srvr, SSL_LOG_ERROR, 
+                        "SSL handshake timed out (client %s, server %s)",
+                        conn->remote_ip, ssl_util_vhostid(conn->pool, srvr));
+                SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
+                while (!SSL_shutdown(ssl));
+                SSL_free(ssl);
+                ap_ctx_set(fb->ctx, "ssl", NULL);
+                ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+                conn->aborted = 1;
+                return;
+            }
+            else if (SSL_get_error(ssl, rc) == SSL_ERROR_SYSCALL) {
+                if (errno == EINTR)
+                    continue;
+                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_ERRNO,
+                        "SSL handshake interrupted by system");
+                SSL_set_shutdown(ssl, SSL_RECEIVED_SHUTDOWN);
+                while (!SSL_shutdown(ssl));
+                SSL_free(ssl);
+                ap_ctx_set(fb->ctx, "ssl", NULL);
+                ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+                conn->aborted = 1;
+                return;
+            }
             else {
                 /*
                  * Ok, anything else is a fatal error
                  */
-                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR,
+                ssl_log(srvr, SSL_LOG_ERROR|SSL_ADD_SSLERR|SSL_ADD_ERRNO,
                         "SSL handshake failed");
+
                 /*
                  * try to gracefully shutdown the connection:
                  * - send an own shutdown message (be gracefully)
@@ -291,7 +331,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
                 while (!SSL_shutdown(ssl));
                 SSL_free(ssl);
                 ap_ctx_set(fb->ctx, "ssl", NULL);
-                fb->flags |= B_EOF|B_EOUT;
+                ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+                conn->aborted = 1;
                 return;
             }
         }
@@ -306,7 +347,8 @@ void ssl_hook_NewConnection(conn_rec *conn)
             while (!SSL_shutdown(ssl));
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
-            fb->flags |= B_EOF|B_EOUT;
+            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+            conn->aborted = 1;
             return;
         }
 
@@ -336,11 +378,18 @@ void ssl_hook_NewConnection(conn_rec *conn)
             while (!SSL_shutdown(ssl));
             SSL_free(ssl);
             ap_ctx_set(fb->ctx, "ssl", NULL);
-            fb->flags |= B_EOF|B_EOUT;
+            ap_bsetflag(fb, B_EOF|B_EOUT, 1);
+            conn->aborted = 1;
             return;
         }
     }
 
+    /*
+     * Remove the timeout handling
+     */
+    ap_set_callback_and_alarm(NULL, 0);
+    ap_ctx_set(ap_global_ctx, "ssl::handshake::timeout", (void *)FALSE);
+
     /*
      * Improve I/O throughput by using
      * SSLeay's read-ahead functionality
@@ -354,6 +403,16 @@ void ssl_hook_NewConnection(conn_rec *conn)
     return;
 }
 
+/*
+ * Signal handler function for the SSL handshake phase
+ */
+void ssl_hook_TimeoutConnection(int sig)
+{
+    /* we just set a flag for the handshake processing loop */
+    ap_ctx_set(ap_global_ctx, "ssl::handshake::timeout", (void *)TRUE);
+    return;
+}
+
 /*
  *  Close the SSL part of the socket connection
  */
@@ -376,8 +435,8 @@ void ssl_hook_CloseConnection(void *_conn)
     /*
      * And finally log the connection close
      */
-    ssl_log(conn->server, SSL_LOG_INFO, "Connection to child %d closed (server %s:%d)",
-            conn->child_num, conn->server->server_hostname, conn->server->port);
+    ssl_log(conn->server, SSL_LOG_INFO, "Connection to child %d closed (server %s)",
+            conn->child_num, ssl_util_vhostid(conn->pool, conn->server));
 
     return;
 }
@@ -421,9 +480,9 @@ int ssl_hook_Handler(request_rec *r)
                               ap_get_server_name(r), thisport);
 
         ap_table_setn(r->notes, "error-notes", ap_psprintf(r->pool,
-                      "You're speaking plain HTTP to an SSL-enabled server.<BR>\n"
-                      "Instead use HTTPS protocol to access this server, please.<BR>\n"
-                      "<BLOCKQUOTE>Hint: <A HREF=\"%s\">%s</A></BLOCKQUOTE>",
+                      "Reason: You're speaking plain HTTP to an SSL-enabled server.<BR>\n"
+                      "Instead use the HTTPS protocol to access this server, please.<BR>\n"
+                      "<BLOCKQUOTE>Hint: <A HREF=\"%s\"><B>%s</B></A></BLOCKQUOTE>",
                       thisurl, thisurl));
         r->status_line = "500 SSL Engine Error";
     }
@@ -452,6 +511,8 @@ int ssl_hook_Auth(request_rec *r)
      */
     if (!sc->bEnabled)
         return DECLINED;
+    if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
+        return DECLINED;
     if (!(dc->nOptions & SSL_OPT_FAKEBASICAUTH))
         return DECLINED;
     if (r->connection->user)
@@ -495,7 +556,7 @@ int ssl_hook_Access(request_rec *r)
     /*
      * Support for SSLRequireSSL directive
      */
-    if (rec->bSSLRequired && !ap_ctx_get(r->connection->client->ctx, "ssl")) {
+    if (rec->bSSLRequired && ap_ctx_get(r->connection->client->ctx, "ssl") == NULL) {
         ap_log_reason("SSL connection required", r->filename, r);
         return FORBIDDEN;
     }
@@ -505,6 +566,8 @@ int ssl_hook_Access(request_rec *r)
      */
     if (!sc->bEnabled)
         return DECLINED;
+    if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
+        return DECLINED;
 
     /*
      * Check SSLRequire boolean expressions
@@ -611,6 +674,8 @@ int ssl_hook_Fixup(request_rec *r)
      */
     if (!sc->bEnabled)
         return DECLINED;
+    if (ap_ctx_get(r->connection->client->ctx, "ssl") == NULL)
+        return DECLINED;
 
     /*
      * Annotate the SSI/CGI environment with standard SSL information
@@ -918,10 +983,10 @@ void ssl_callback_LogTracingState(SSL *ssl, int where, int rc)
         }
         else if (where & SSL_CB_EXIT) {
             if (rc == 0)
-                ssl_log(s, SSL_LOG_TRACE, "%s: Exit: failed in %s\n", 
+                ssl_log(s, SSL_LOG_TRACE, "%s: Exit: failed in %s", 
                         SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
             else if (rc < 0)
-                ssl_log(s, SSL_LOG_TRACE, "%s: Exit: error in %s\n", 
+                ssl_log(s, SSL_LOG_TRACE, "%s: Exit: error in %s", 
                         SSL_LIBRARY_NAME, SSL_state_string_long(ssl));
         }
     }
