@@ -9,7 +9,7 @@
 */
 
 /* ====================================================================
- * Copyright (c) 1998 Ralf S. Engelschall. All rights reserved.
+ * Copyright (c) 1998-1999 Ralf S. Engelschall. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -609,9 +609,9 @@ static char *ssl_var_lookup_ssl_version(pool *p, char *var)
     }
     else if (strEQ(var, "LIBRARY")) {
         result = ap_pstrdup(p, SSLeay_version(SSLEAY_VERSION));
-        if ((cp = index(result, ' ')) != NULL) {
+        if ((cp = strchr(result, ' ')) != NULL) {
             *cp = '/';
-            if ((cp2 = index(cp, ' ')) != NULL)
+            if ((cp2 = strchr(cp, ' ')) != NULL)
                 *cp2 = NUL;
         }
     }
